unit Table;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	Clipbrd, Consts, Grids;

type
	TfrmTable = class(TForm)
		grdMain: TStringGrid;
		procedure FormCreate(Sender: TObject);
		procedure FormResize(Sender: TObject);
    procedure grdMainKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
	public
		ChartForm: TForm;
		procedure OptimizeWindowSize ();
		procedure LoadFile (const FileName: string);
	private
		procedure set_grid_cell (r,c: integer; var S: string);
		procedure CopyToClipboard ();
	end;

var
	frmTable: TfrmTable;

implementation

{$R *.DFM}

const
	gsValue = 0;
	gsEoln = 1;
	gsEof = 2;

procedure TfrmTable.FormCreate(Sender: TObject);
begin
	//ChartForm:= nil;
end;

procedure TfrmTable.FormResize(Sender: TObject);
begin
	if not assigned(ChartForm) then exit;
	ChartForm.Top:= Top;
	ChartForm.Left:= Left-ChartForm.Width;
end;

procedure TfrmTable.OptimizeWindowSize ();
	var c, w, wmax: integer;
begin
	w:= 0;
	for c:= 0 to grdMain.ColCount-1 do
		w:= w + grdMain.ColWidths[c] + grdMain.GridLineWidth;
	wmax:= Round(Screen.Width*0.8);
	if w>wmax then w:= wmax;
	ClientWidth:= w + 17;
end;

procedure TfrmTable.set_grid_cell (r,c: integer; var S: string);
var
	w: integer;
begin
	with grdMain do begin
		if ColCount<=c then ColCount:= c+1;
		if RowCount<=r then RowCount:= r+1;
		Cells[c,r]:= S;
		w:= Canvas.TextWidth(S)+5;
		if ColWidths[c]<w then ColWidths[c]:= w;
	end;
end;

function ReadCell (var F: text; var buf: string; delim: char): integer;
var
	ch: char;
	N: integer;
	quoted: boolean;
begin
	ReadCell:= gsValue;
	if EOF(F) then begin
		ReadCell:= gsEof; buf:= '';
	end
	else begin
		N:= 0; ch:= '?'; quoted:= false;
		SetLength (buf,N);
		while not eoln(F) and ((ch<>delim) or quoted) do begin
			read (F,ch);
			if (ch='"') and quoted then quoted:= false;
			if ((ch<>delim) or quoted) and (ch<>'"') then begin
				N:= N+1;
				SetLength (buf,N);
				buf[N]:= ch;
			end;
			if (N=0) then quoted:= (ch='"');
		end;
		if eoln(F) then begin
			readln(F);
			ReadCell:= gsEoln;
		end;
	end;
end;

procedure TfrmTable.LoadFile (const FileName: string);
var txt: TextFile;
	c, r, state: integer;
	S, Sr: string;
begin
	{$I-}
	AssignFile(txt, FileName);
	Reset(txt);
	{$I+}
	if (IOResult = 0) and (FileName <> '') then begin
		if grdMain.ColCount<2 then grdMain.ColCount:= 2;
		if grdMain.RowCount<2 then grdMain.RowCount:= 2;
		grdMain.FixedCols:= 1;
		grdMain.FixedRows:= 1;
		c:= 0; r:= 0;
		repeat
			state:= ReadCell (txt, S, #9);
			if (state=gsEof) and (Length(S)=0) then
				Break;
			if (c=0) and (r=0) and (Length(S)>0) then
				grdMain.FixedCols:= 0;
			if (c=0) and (r>0) and (grdMain.FixedCols>0) then begin
				str(r,Sr);
				set_grid_cell (r,c,Sr);
				c:= 1;
			end;
			set_grid_cell (r,c,S);
			Inc(c);
			if state=gsEoln then begin
				c:= 0; Inc(r);
			end;
		until state=gsEof;
		CloseFile(txt);
		grdMain.RowCount:= r;
		OptimizeWindowSize;
	end;
end;

procedure TfrmTable.CopyToClipboard ();
var
	c, r, Len, i: integer;
	buf: array of char;
	GR: TGridRect;
	S: string;
begin
	GR:= grdMain.Selection;
	//    
	Len:= 1;
	for r:= GR.Top to GR.Bottom do begin
		for c:= GR.Left to GR.Right do begin
			Inc(Len, Length(grdMain.Cells[c,r])+1);
		end;
		Inc(Len);
	end;
	SetLength (buf, Len);
	//  
	Len:= 0;
	for r:= GR.Top to GR.Bottom do begin
		for c:= GR.Left to GR.Right do begin
			S:= grdMain.Cells[c,r];
			for i:= 1 to Length(S) do begin
				buf[Len]:= S[i];
				Inc(Len);
			end;
			if c<GR.right then begin
				buf[Len]:= #9;
				Inc(Len);
			end;
		end;
		buf[Len]:= #13;
		buf[Len+1]:= #10;
		Inc (Len,2);
	end;
	buf[Len]:= #0;
	Clipboard.Clear;
	Clipboard.SetTextBuf (PChar(buf));
end;

procedure TfrmTable.grdMainKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
	GR: TGridRect;
begin
	if (Shift=[ssCtrl]) then case Key of
		ord('A') : begin
			GR.Left:= grdMain.FixedCols;
			GR.Right:= grdMain.ColCount-1;
			GR.Top:= 0;
			GR.Bottom:= grdMain.RowCount-1;
			grdMain.Selection:= GR;
			end;
		ord('C'), VK_INSERT : begin
			CopyToClipboard;
			end;
		end;
end;

end.


