unit SliChart;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls,
	Clipbrd, Printers, Forms, Consts, Grids;

const
	CL_DATA_NODATA = 0; //   
	CL_DATA_BYTE = 1;   //  
	CL_DATA_REAL = 2;   //  
	CL_DATA_TEXT = 3;   //  char*

	//   
	CL_EMPTY_BYTE = -32768;
	CL_EMPTY_REAL = 1.6E307;
	CL_EMPTY_TEXT = '???';

	//   ,   TChartBackInfo.event
	CL_CBACK_DATACHANGE = 1;   //  
	CL_CBACK_ZOOM = 2;   // 
	CL_CBACK_POINTACTIVATE = 3;   //  
	CL_CBACK_POINTSELECT = 4;   //      
	CL_CBACK_POINTHIGHLIGHT = 5;   //  
	CL_CBACK_SCENECHANGE = 6;   //   3D-
	CL_CBACK_STATUSCHANGE = 7;   //   
	CL_CBACK_SLIDECHANGE = 8;   //   

type
	//      
	TInputData = record
		unused: integer;
		case dType: integer of
			CL_DATA_BYTE: (dByte: Smallint);  //  
			CL_DATA_REAL: (dReal: double); //  
			CL_DATA_TEXT: (dText: PChar);  //  char*
	end;

	// ,       
	TPointInfo = record
		panel_name: PChar;   //  
		series_name: PChar;   //  
		series: integer;   //    
		point: integer;   //    
		X, Y, Z: double;   //  
		text: PChar;   //   
		selected: integer; // ?
		can_be_selected: integer; //    ?
	end;

	//   
	TZoom = record
		x1, y1, z1: double;  //         
		x2, y2, z2: double;  //        
	end;

	//       
	TSliChart = class;
	TChartBackInfo = record
		slide: TSliChart;   //    ,  
		event: integer;   //  
		text: PChar;   //  ,  ,   
		area: PChar;   //   ,    
		series: integer;   //   
		point: integer;   //    
		selected: integer;   //  ?
		prev: TZoom;   //    
		curr: TZoom;   //    
		prev_slide: integer;   //     
		curr_slide: integer;   //     
	end;

	//    ChartLib
	TChartEvent = procedure (Sender: TObject; var Info: TChartBackInfo) of object;

	//        
	FMatrixInterface = function (DataHolder: TObject; row, col: integer; var data: TInputData): integer; cdecl;

	//   
	TSelectShape = (shNone, shRectangle, shEllipse, shHalf, shStrip);

	//   
	TSelectMode = (smNone, smSelect, smUnselect, smInvert);

	//    - -   ChartLib
	TSliChart = class (TCustomControl)
	private
		slide: integer;   //  ,   ChartLib
		select_mode: TSelectMode;   //    
		select_shape: TSelectShape;   //    
		select_outside: boolean;   //    ?
		hastatus: boolean;   //     ?
		last_error_code: integer;   //      ChartLib
		// 
		win_drawer: integer;   //    
		now_continue_draw: boolean;   //   ?
		now_demo_loaded: boolean;   //    ?
		now_internal_draw: boolean;   //   ?
		now_total_redraw: boolean;   //      
		//   
		buffered: boolean;   //   (published HasBuffer)?
		buffer_complete: boolean;   //  ?
		buffer_bitmap: HBITMAP;   //   
		buffer_old_bitmap: HBITMAP;   //    
		buffer_mem_dc: HDC;   //  ,    
		buffer_drawer: integer;   //    
		buffer_width: integer;   //     
		buffer_height: integer;   //     
		//   
		grid: TStringGrid;
		grid_series_no: array of integer;
		grid_point_no: array of integer;
		grid_lines: integer;
		grid_has_numbers: boolean;
		//  
		FOnPaintStart: TNotifyEvent;
		FOnPaintFinish: TNotifyEvent;
		FOnDataChange: TChartEvent;
		FOnSlideChange: TChartEvent;
		FOnDataFile: TChartEvent;
		FOnZoom: TChartEvent;
		FOnPointActivate: TChartEvent;
		FOnPointSelect: TChartEvent;
		FOnPointHighlight: TChartEvent;
		FOnSceneChange: TChartEvent;
		FOnStatusChange: TChartEvent;
		FOnPropertyChange: TNotifyEvent;
		FOnShowTable: TNotifyEvent;
		//  
		procedure reset_draw ();
		procedure reset_draw_window ();
		procedure reset_draw_buffer ();
		procedure begin_draw ();
		procedure continue_draw ();
		procedure end_draw ();
		procedure demo_draw ();
		procedure buffer_create ();
		procedure buffer_free ();
		procedure buffer_show ();
		procedure draw_metafile (var SlideshotEMF: TMetaFile);
		procedure grid_clear ();
		procedure grid_set_cell (r,c: integer; var S: string);
		procedure grid_add_line (line: PChar; nseries: integer; npoint: integer);
		procedure grid_find_line (nseries: integer; npoint: integer);
		procedure arrow_key (Key: Word);
		//  
		procedure GridSelectCell (Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
		procedure SetGrid (set_grid: TStringGrid);
		procedure SetHasStatus (set_hastatus: boolean);
		procedure SetHasBuffer (set_buffered: boolean);
		procedure SetSelectOutside (set_select_outside: boolean);
		procedure SetSelectMode (set_select_mode: TSelectMode);
		procedure SetSelectShape (set_select_shape: TSelectShape);
		//  
		procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
		procedure CNKeyDown(var Message: TWMKeyDown); message CN_KEYDOWN;
		//procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		// ,  
		procedure DataFile (const FileName: PChar);
		procedure DataLine (const Line: PChar);
		procedure DataVector (var Data: array of TInputData);
		procedure DataMatrix (rows: integer; fields: PChar; func: FMatrixInterface; obj: TObject);
		procedure DataGet (row: integer; colname: PChar; var data: TInputData);
		procedure DataPut (row: integer; colname: PChar; var data: TInputData);
		procedure SlideSetRect ();
		procedure SlideMaxZoom ();
		procedure SlideRefresh ();
		procedure SlideDraw ();
		procedure ChartScreenToReal (x, y: integer; var PX, PY, PZ: double);
		procedure ChartRealToScreen (xreal, yreal: double; var x, y: integer);
		procedure ChartGetZoom (var x1: double; var y1: double; var x2: double; var y2: double);
		procedure ChartGetMaxZoom (var x1: double; var y1: double; var x2: double; var y2: double);
		procedure ChartSetZoom (x1: double; y1: double; x2: double; y2: double);
		procedure ChartDrawPoint (var info: TPointInfo; stage: integer);
		procedure ChartSelectPoint (var info: TPointInfo);
		procedure ChartSelectAll ();
		procedure ChartSetActivePoint (var info: TPointInfo; dir: integer);
		procedure ChartGetActivePoint (var info: TPointInfo);
		procedure ChartGetPointInfo (var info: TPointInfo);
		procedure ChartGetTableLine (var info: TPointInfo);
		procedure ChartGetXYInfo (x, y: integer; series, expr: PChar; var buffer: PChar);
		//  ,  
		procedure SaveAsEMF (const filename: string);
		procedure SaveAsBMP (const filename: string);
		procedure SaveAsGIF (const filename: string);
		procedure CopyToClipboard ();
		procedure Print ();
		procedure ShowTable ();
		//   
		procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure MouseMove(Shift: TShiftState; X,Y: Integer); override;
		procedure KeyDown(var Key: Word; Shift: TShiftState); override;
		procedure Paint(); override;
		//     
		property SlideNo: integer read slide;
    property Busy: boolean read now_continue_draw;
    property ChartLibError: integer read last_error_code;
	published
		//   
		property HasBuffer: boolean read buffered write SetHasBuffer;
		property HasStatus: boolean read hastatus write SetHasStatus;
		property SelectOutside: boolean read select_outside write SetSelectOutside;
		property SelectMode: TSelectMode read select_mode write SetSelectMode;
		property SelectShape: TSelectShape read select_shape write SetSelectShape;
		property SelectedPointsGrid: TStringGrid read grid write SetGrid;
		//   
		property OnPaintStart: TNotifyEvent read FOnPaintStart write FOnPaintStart;
		property OnPaintFinish: TNotifyEvent read FOnPaintFinish write FOnPaintFinish;
		property OnStatusChange: TChartEvent read FOnStatusChange write FOnStatusChange;
		property OnDataChange: TChartEvent read FOnDataChange write FOnDataChange;
		property OnSlideChange: TChartEvent read FOnSlideChange write FOnSlideChange;
		property OnDataFile: TChartEvent read FOnDataFile write FOnDataFile;
		property OnZoom: TChartEvent read FOnZoom write FOnZoom;
		property OnPointActivate: TChartEvent read FOnPointActivate write FOnPointActivate;
		property OnPointSelect: TChartEvent read FOnPointSelect write FOnPointSelect;
		property OnPointHighlight: TChartEvent read FOnPointHighlight write FOnPointHighlight;
		property OnSceneChange: TChartEvent read FOnSceneChange write FOnSceneChange;
		property OnPropertyChange: TNotifyEvent read FOnPropertyChange write FOnPropertyChange;
		property OnShowTable: TNotifyEvent read FOnShowTable write FOnShowTable;
		//  ,   TCustomControl
		property Align;
		property Anchors;
		property AutoSize;
		property ClientHeight;
		property ClientWidth;
		property Color;
		property Constraints;
		property PopupMenu;
		property Visible;
		//  ,   TCustomControl
		property OnCanResize;
		property OnClick;
		property OnConstrainedResize;
		property OnDblClick;
		property OnDockDrop;
		property OnDockOver;
		property OnDragDrop;
		property OnDragOver;
		property OnEndDock;
		property OnGetSiteInfo;
		property OnKeyDown;
		property OnKeyPress;
		property OnKeyUp;
		property OnMouseDown;
		property OnMouseUp;
		property OnMouseMove;
		property OnMouseWheel;
		property OnMouseWheelDown;
		property OnMouseWheelUp;
		property OnResize;
		property OnStartDock;
		property OnUnDock;
	end;

procedure SetLogName (filename: PChar);   //   
procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('Samples', [TSliChart]);
end;

(*********************************************************************
	    CHARTLIB.DLL
**********************************************************************)
const
	//   
	CL_DRAWER_GIF = 3;  // GIF-
	CL_DRAWER_WIN = 4;  //    ,   Windows    (HDC)
	CL_DRAWER_WMF = 5;  // Windows 
	CL_DRAWER_EMF = 6;  // Enhanced Windows 
	//  
	CL_STAGE_BEGIN = 0;      //  
	CL_STAGE_BACKGROUND = 1; //  , , ,  
	CL_STAGE_FILLS = 2;      // 
	CL_STAGE_BOXES = 3;      // 
	CL_STAGE_HISTOGRAMS = 4; // 
	CL_STAGE_CANDLES = 5;    // 
	CL_STAGE_LINES = 6;      //    
	CL_STAGE_POINTS = 7;     // 
	CL_STAGE_ACTPOINT = 8;   //  
	CL_STAGE_FINDPOINT = 9;  //    
	CL_STAGE_STATUS = 10;    //        
	CL_STAGE_DYNAMENTS = 11; //  
	CL_STAGE_END = 12;       // 

type
TDrawerDef = record
	drtype: integer;  //  
	filename: PChar;  //    GPT, GIF, WMF
	fout: pointer;   // FILE*
	fhandle: integer;   //    
	x1, y1, x2, y2: integer;   //    
	hdc: HDC;  //    Windows-
end;

//   
FChartCallBack = function (var info: TChartBackInfo): integer; cdecl;

// ---------------------------------------------------------------------
//  
var
	cmd: array [0..300] of char;
	poinfo: TPointInfo;
	ddef: TDrawerDef;

// ---------------------------------------------------------------------
//   

//  
function CL_WorkStart (): integer;
	cdecl; external 'ChartLib.DLL';

//  
function CL_WorkFinish (): integer;
	cdecl; external 'ChartLib.DLL';

//     , filename=0  
function CL_WorkDebug (filename: PChar; ErrorsHandler: pointer=nil): integer;
	cdecl; external 'ChartLib.DLL';

// ---------------------------------------------------------------------
//    

//      type
//  ,    clDrawerUse
function CL_DrawerCreate (var data: TDrawerDef): integer;
	cdecl; external 'ChartLib.DLL';

//   
function CL_DrawerUpdate (var data: TDrawerDef): integer;
	cdecl; external 'ChartLib.DLL';

//     drawerID
function CL_DrawerUse (drawer: integer): integer;
	cdecl; external 'ChartLib.DLL';

	//     drawerID
function CL_DrawerDelete (drawer: integer): integer;
	cdecl; external 'ChartLib.DLL';

//      (      ).
function CL_DrawerSave (): integer;
	cdecl; external 'ChartLib.DLL';


// ---------------------------------------------------------------------
//    

//     ,  #Slide, #Chart  .     slide
function CL_DataLine (var slide: integer; line: PChar): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_DataVector (slide: integer; var data: TInputData): integer;
	cdecl; external 'ChartLib.DLL';

//          
function CL_DataFile (var slide: integer; filename: PChar): integer;
	cdecl; external 'ChartLib.DLL';

//        rows*cols   fields
function CL_DataMatrix (slide, rows: integer; fields: PChar; func: FMatrixInterface; obj: TObject): integer;
	cdecl; external 'ChartLib.DLL';

//        point     colname
function CL_DataGetValue (slide, row: integer; colname: PChar;
	var data: TInputData): integer;
	cdecl; external 'ChartLib.DLL';

//        point     colname
function CL_DataSetValue (slide, row: integer; colname: PChar;
	var data: TInputData): integer;
	cdecl; external 'ChartLib.DLL';

// ---------------------------------------------------------------------
//   

//      
function CL_SlideSetRect (slide: integer;
	xleft: integer; ytop: integer; xright: integer; ybottom: integer): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_SlideMaxZoom (slide: integer): integer;
	cdecl; external 'ChartLib.DLL';

//  ()  (  ,  ,   ..)
function CL_SlideRefresh (slide: integer): integer;
	cdecl; external 'ChartLib.DLL';

//   
function CL_SlideDraw (slide: integer): integer;
	cdecl; external 'ChartLib.DLL';

//      stage
function CL_SlideBeginDraw (slide: integer;
	from_stage: integer=CL_STAGE_BEGIN): integer;
	cdecl; external 'ChartLib.DLL';

//      stage 
function CL_SlideContinueDraw (slide: integer;
	to_stage: integer=CL_STAGE_END): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_SlideCallBack (slide: integer;
	callback: FChartCallBack; recepient: TSliChart): integer;
	cdecl; external 'ChartLib.DLL';
{
//      area 
function CL_SlideGetSize (slide: integer; area: integer;
	var xleft: integer; var ytop: integer; var xright: integer; var ybottom: integer): integer;
	cdecl; external 'ChartLib.DLL';
}

// ---------------------------------------------------------------------
//   
//     
function CL_ChartScreenToReal (slide, x, y: integer; var PX, PY, PZ: double): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_ChartRealToScreen (slide: integer; xreal, yreal: double; var x, y: integer): integer;
	cdecl; external 'ChartLib.DLL';

//        series_name    (x,y)
function CL_ChartGetXYInfo (slide, x, y: integer; series, expr: PChar; var buffer: PChar): integer;
	cdecl; external 'ChartLib.DLL';

//        
function CL_ChartGetZoom (slide: integer;
	var x1: double; var y1: double; var x2: double; var y2: double): integer;
	cdecl; external 'ChartLib.DLL';

//        
function CL_ChartGetMaxZoom (slide: integer;
	var x1: double; var y1: double; var x2: double; var y2: double): integer;
	cdecl; external 'ChartLib.DLL';

//       
function CL_ChartSetZoom (slide: integer;
	x1: double; y1: double; x2: double; y2: double): integer;
	cdecl; external 'ChartLib.DLL';

// ---------------------------------------------------------------------
//     

//   
function CL_ChartDrawPoint (slide: integer;
	var info: TPointInfo; stage: integer): integer;
	cdecl; external 'ChartLib.DLL';

//  , selectmode= 1- , 2-, 3- 
function CL_ChartSelectPoint (slide: integer; var info: TPointInfo; selectmode: integer): integer;
	cdecl; external 'ChartLib.DLL';

//   "  "
function CL_ChartSelectAll (slide, selectmode: integer): integer;
	cdecl; external 'ChartLib.DLL';

//   point   series.
function CL_ChartSetActivePoint (slide: integer;
	var info: TPointInfo; dir: integer): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_ChartGetActivePoint (slide: integer; var info: TPointInfo): integer;
	cdecl; external 'ChartLib.DLL';

//    
function CL_ChartGetPointInfo (slide: integer; var info: TPointInfo): integer;
	cdecl; external 'ChartLib.DLL';

//       
function CL_ChartGetTableLine (slide: integer; var info: TPointInfo): integer;
	cdecl; external 'ChartLib.DLL';

// ---------------------------------------------------------------------
//     

//   
function CL_OnMouseDown (slide, x, y, buttons: integer): integer;
	cdecl; external 'ChartLib.DLL';

function CL_OnMouseMove (slide, x, y, buttons: integer): integer;
	cdecl; external 'ChartLib.DLL';

function CL_OnMouseUp   (slide, x, y, buttons: integer): integer;
	cdecl; external 'ChartLib.DLL';

//     
function CL_OnKeyPress (slide, keys, buttons: integer): integer;
	cdecl; external 'ChartLib.DLL';

// ---------------------------------------------------------------------
//  
function call_back (var info: TChartBackInfo): integer; cdecl;
var
	userdef: TChartEvent;
begin
	Result:= 0;
	userdef:= nil;
	if not Assigned(@info) or not (info.slide is TSliChart) then exit;
	with info.slide do begin
		case info.event of
			CL_CBACK_DATACHANGE:   //  
				userdef:= FOnDataChange;
			CL_CBACK_SLIDECHANGE:   //   
				userdef:= FOnSlideChange;
			CL_CBACK_ZOOM: begin  // 
				//   
				SlideRefresh;
				userdef:= FOnZoom;
				end;
			CL_CBACK_POINTACTIVATE:   //  
				userdef:= FOnPointActivate;
			CL_CBACK_POINTSELECT: begin  //      
				if buffered then begin
					//       
					reset_draw_buffer;
					poinfo.series:= info.series;
					poinfo.point:= info.point;
					CL_ChartDrawPoint (slide, poinfo, CL_STAGE_BOXES);
					CL_ChartDrawPoint (slide, poinfo, CL_STAGE_HISTOGRAMS);
					CL_ChartDrawPoint (slide, poinfo, CL_STAGE_CANDLES);
					CL_ChartDrawPoint (slide, poinfo, CL_STAGE_POINTS);
					reset_draw;
					end;
				userdef:= FOnPointSelect;
				end;
			CL_CBACK_POINTHIGHLIGHT:   //  
				userdef:= FOnPointHighlight;
			CL_CBACK_SCENECHANGE:   //   3D-
				userdef:= FOnSceneChange;
			CL_CBACK_STATUSCHANGE:   //   
				userdef:= FOnStatusChange;
			end;
		//   
		if (@userdef=nil) or now_internal_draw then exit;
		userdef (info.slide, info);
		reset_draw;
		end;
	Result:= 1;
end;

(*********************************************************************
	   --     CHARTLIB
**********************************************************************)

//  -
constructor TSliChart.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	//ControlStyle:= ControlStyle + [csReplicatable] + [csOpaque];
	if CL_WorkStart=0 then begin
		//   floating-point 
		Set8087CW ($133F);
		end;
	win_drawer:= -1;
	slide:= 0;
	now_continue_draw:= false;
	now_demo_loaded:= false;
	now_internal_draw:= false;
	now_total_redraw:= false;
	buffer_drawer:= -1;
	buffer_bitmap:= 0;
	buffer_old_bitmap:= 0;
	buffer_mem_dc:= 0;
	buffer_complete:= false;
	buffer_width:= 0;
	buffer_height:= 0;
	last_error_code:= 0;
	if csDesigning in ComponentState then
		Exit;
	DataLine ('#NewSlide default');
	DataLine ('#SetFilm default');
	CL_SlideCallBack (slide, call_back, Self);
	//DataLine ('#DumpEcho on');
	//DataLine (StrFmt (cmd, '#Dump NewSlide==1==%d', [SlideNo]));
end;

//  -
destructor TSliChart.Destroy;
begin
	CL_DrawerDelete (win_drawer);
	if buffered then begin
		CL_DrawerDelete (buffer_drawer);
		buffer_free;
		end;
	CL_WorkFinish;
	inherited Destroy;
end;

//   
procedure TSliChart.demo_draw();
begin
	reset_draw_window;
	if now_demo_loaded then begin
		DataLine (StrFmt (cmd, '#Slide design_time_%s', [Name]));
    end
	else begin
		DataLine (StrFmt (cmd, '#NewSlide design_time_%s', [Name]));
		DataLine ('#SetFilm design_time_film');
		DataLine ('#Slide.Background 7 20 20 7');
		DataLine ('#ExtraY 0');
		DataLine ('#Format X=(@/10-1.5) Y=(X^3-X) S=3 C=7 LC=23 LW=2 FC=20 FG=7 FI=X FD=4');
		DataLine ('#SeriesSize 30');
		DataLine ('#Title.Justify left');
		DataLine (StrFmt (cmd, '#Title %s', [Name]));
		now_demo_loaded:= true;
		end;
	CL_SlideMaxZoom (slide);
	SlideSetRect;
	CL_SlideRefresh (slide);
	CL_SlideDraw (slide);   //  
end;

// ---------------------------------------------------------------------
//   

//    
procedure TSliChart.reset_draw_window ();
begin
	ddef.drtype:= CL_DRAWER_WIN;
	ddef.hdc:= Canvas.Handle;
	if win_drawer<0 then
		win_drawer:= CL_DrawerCreate (ddef);
	CL_DrawerUse (win_drawer);
	CL_DrawerUpdate (ddef);
	CL_SlideCallBack (slide, call_back, Self);
	SlideSetRect;
end;

//    
procedure TSliChart.reset_draw_buffer ();
begin
	ddef.drtype:= CL_DRAWER_WIN;
	ddef.hdc:= buffer_mem_dc;
	if buffer_drawer<0 then
		buffer_drawer:= CL_DrawerCreate (ddef);
	CL_DrawerUse (buffer_drawer);
	CL_DrawerUpdate (ddef);
	CL_SlideCallBack (slide, call_back, Self);
	SlideSetRect;
end;

//      
procedure TSliChart.reset_draw ();
begin
	if now_internal_draw then exit;
	//   ,        
	if buffered and (buffer_mem_dc<>0) and not buffer_complete then
		reset_draw_buffer
	else
		reset_draw_window;
end;

//  
procedure TSliChart.buffer_create ();
var
	DC: HDC;
begin
	buffer_free;
	DC := GetDC (0);
	buffer_bitmap:= CreateCompatibleBitmap (DC, ClientWidth, ClientHeight);
	ReleaseDC (0, DC);
	buffer_mem_dc:= CreateCompatibleDC (0);
	buffer_old_bitmap:= SelectObject (buffer_mem_dc, buffer_bitmap);
	buffer_width:= ClientWidth;
	buffer_height:= ClientHeight;
	buffer_complete:= false;
end;

//  
procedure TSliChart.buffer_free ();
begin
	if (buffer_mem_dc<>0) and (buffer_old_bitmap<>0) then
		SelectObject (buffer_mem_dc, buffer_old_bitmap);
	if (buffer_mem_dc<>0) then
		DeleteDC (buffer_mem_dc);
	if (buffer_bitmap<>0) then
		DeleteObject (buffer_bitmap);
	buffer_mem_dc:= 0;
	buffer_bitmap:= 0;
	buffer_old_bitmap:= 0;
end;

//  
procedure TSliChart.buffer_show ();
begin
	if buffered and (buffer_mem_dc<>0) then begin
		BitBlt (
			Canvas.Handle, 0, 0, ClientWidth, ClientHeight,
			buffer_mem_dc, 0, 0, SRCCOPY);
		buffer_complete:= true;
		//        8
		reset_draw_window;
		CL_SlideBeginDraw (slide, CL_STAGE_ACTPOINT);
		end;
end;

//    
procedure TSliChart.begin_draw ();
begin
	if not Visible
		or (csDesigning in ComponentState)
		or (ClientWidth<2) or (ClientHeight<2) then Exit;
	//     ,    
	buffer_complete:= buffer_complete
		and buffered
		and (buffer_mem_dc<>0)
		and (buffer_width=ClientWidth)
		and (buffer_height=ClientHeight);
	//     
	if buffer_complete then begin
		//    
		buffer_show;
	end
	else begin
		//   -    
		if Assigned(FOnPaintStart) then
			FOnPaintStart (self);
		//     
		reset_draw_window;
		CL_SlideRefresh (slide);
		CL_SlideBeginDraw (slide);
		while CL_SlideContinueDraw (slide, CL_STAGE_BACKGROUND)=1 do;
		//     
		if buffered then begin
			buffer_create;
  		reset_draw_buffer;
			CL_SlideBeginDraw (slide);
		end;
    now_total_redraw:= true;
  end;
	//     
	continue_draw;
end;

//   
procedure TSliChart.continue_draw ();
begin
	if not Visible then Exit;
	now_continue_draw:= true;
	InvalidateRect (Handle, nil, FALSE);
end;

//   
procedure TSliChart.end_draw();
var
	old_total_redraw: boolean;
begin
	now_continue_draw:= false;
  old_total_redraw:= now_total_redraw;
	now_total_redraw:= false;
	//   -    
	if old_total_redraw and Assigned(FOnPaintFinish) then
		FOnPaintFinish (self);
end;

//  Windows- " "
procedure TSliChart.Paint();
begin
	if not Visible then Exit;
	if csDesigning in ComponentState then
		demo_draw
	else if now_continue_draw then begin
		reset_draw;
		//      7- ,      
		if buffered and not buffer_complete then begin
			now_continue_draw:= (CL_SlideContinueDraw(slide,CL_STAGE_POINTS)=1);
			//    -      
			if not now_continue_draw then
				buffer_show;
      end;
    //  
    now_continue_draw:= (CL_SlideContinueDraw(slide)=1);
		if now_continue_draw then
			continue_draw   //    WM_PAINT
    else
      end_draw;
		end
  //    ,  
	else begin
		inherited Paint;
		end;
end;

//  Windows- "  "
procedure TSliChart.WMEraseBkgnd (var Message: TWMEraseBkgnd);
begin
	begin_draw;
	Message.Result:= 1;
end;

// ---------------------------------------------------------------------
//     

//  Windows-     ChartLib
function get_buttons (Shift: TShiftState): integer;
var
	s: integer;
begin
	s:= 0;
	if ssShift in Shift then s:= s+1;
	if ssAlt in Shift then s:= s+2;
	if ssCtrl in Shift then s:= s+4;
	if ssLeft in Shift then s:= s+8;
	if ssRight in Shift then s:= s+16;
	if ssMiddle in Shift then s:= s+32;
	if ssDouble in Shift then s:= s+64;
	get_buttons:= s;
end;

//  Windows-    ChartLib
function get_key (key: Word): integer;
begin
	get_key:= 0;
	case Key of
		VK_LEFT  : get_key:= 1;
		VK_RIGHT : get_key:= 2;
		VK_UP    : get_key:= 3;
		VK_DOWN  : get_key:= 4;
		VK_PRIOR : get_key:= 5;
		VK_NEXT  : get_key:= 6;
		VK_HOME	 : get_key:= 7;
		VK_END   : get_key:= 8;
		VK_TAB   : get_key:= 9;
		VK_RETURN: get_key:= 10;
 		VK_INSERT: get_key:= 11;
 		VK_DELETE: get_key:= 12;
 		VK_BACK  : get_key:= 13;
 		VK_SPACE : get_key:= 14;
 		VK_ESCAPE: get_key:= 15;
		end;
end;

//    
procedure TSliChart.MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if not Visible then Exit;
	if buffered and (buffer_mem_dc<>0) and not buffer_complete then Exit;
	reset_draw;
	CL_OnMouseDown (slide, X, Y, get_buttons(Shift));
	inherited MouseDown (Button, Shift, X, Y);
end;

//   
procedure TSliChart.MouseMove	(Shift: TShiftState; X,Y: Integer);
begin
	if not Visible then Exit;
	if buffered and (buffer_mem_dc<>0) and not buffer_complete then Exit;
	reset_draw;
	CL_OnMouseMove (slide, X, Y, get_buttons(Shift));
	inherited MouseMove (Shift, X, Y);
	continue_draw;
end;

//    
procedure TSliChart.MouseUp
	(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	info: TPointInfo;
begin
	if not Visible then Exit;
	if buffered and (buffer_mem_dc<>0) and not buffer_complete then Exit;
	reset_draw;
	CL_OnMouseUp (slide, X, Y, get_buttons(Shift));
	if (CL_ChartGetActivePoint(slide,info)=0) then
		grid_find_line (info.series, info.point);
	inherited MouseUp (Button, Shift, X, Y);
	continue_draw;
end;

//   
procedure TSliChart.KeyDown (var Key: Word; Shift: TShiftState);
var
	slide_old: integer;
begin
	inherited KeyDown (Key, Shift);
	if not Visible then Exit;
	if buffered and (buffer_mem_dc<>0) and not buffer_complete then Exit;
	slide_old:= slide;
	//     
	reset_draw;
	if CL_OnKeyPress (slide, get_key(Key), get_buttons(Shift))=1 then
		continue_draw
	//   Ctrl-
	else if (Shift=[ssCtrl]) then case Key of
		VK_NEXT :
			DataLine ('#Slide <');
		VK_PRIOR :
			DataLine ('#Slide >');
		ord('R') :
			SlideRefresh;
		ord('A') :
			CL_ChartSelectAll (slide, Ord(select_mode));
		ord('M') :
			SelectMode:= TSelectMode ((Ord(select_mode)+1) mod 4);
		ord('S') :
			SelectShape:= TSelectShape ((Ord(select_shape)+1) mod 5);
		ord('C'), VK_INSERT :
			CopyToClipboard;
		ord('P') :
			Print;
		ord('T') :
    	ShowTable;
    end
	//    ( Ctrl, Shift, Alt)
	else if (Shift=[]) then case Key of
		VK_RIGHT, VK_LEFT, VK_UP, VK_DOWN:
			arrow_key (Key);
		end;
	//   
	if slide<>slide_old then
    SlideRefresh;
end;

//   
procedure TSliChart.arrow_key (Key: Word);
var dir, res: integer;
begin
	if CL_ChartGetActivePoint (slide,poinfo)=0 then begin
		dir:= 0;
		case Key of
			VK_RIGHT : dir:= +1;
			VK_LEFT  : dir:= -1;
			VK_UP    : dir:= +2;
			VK_DOWN  : dir:= -2;
		end;
		res:= CL_ChartSetActivePoint (slide,poinfo,dir);
		if res=0 then
			grid_find_line (poinfo.series, poinfo.point);
	end;
end;

//     CN_KEYDOWN,
//       KeyDown
procedure TSliChart.CNKeyDown (var Message: TWMKeyDown);
begin
	Message.Result:= 0;
end;

// ---------------------------------------------------------------------
//   

procedure TSliChart.draw_metafile (var SlideshotEMF: TMetaFile);
var
	SlideshotEMFCanvas: TMetafileCanvas;
	EmfDrawerID: integer;
begin
	//  
	SlideshotEMF:= TMetafile.Create;
	SlideshotEMF.Width:= ClientWidth;
	SlideshotEMF.Height:= ClientHeight;
	//   
	SlideshotEMFCanvas:= TMetafileCanvas.Create (SlideshotEMF,0);
	//   ChartLib   
	ddef.drtype:= CL_DRAWER_EMF;
	ddef.hdc:= SlideshotEMFCanvas.Handle;
	EmfDrawerID:= CL_DrawerCreate (ddef);
	if EmfDrawerID<0 then exit;
	//  
	now_internal_draw:= true;
	CL_SlideRefresh (slide);
	last_error_code:= CL_SlideDraw (slide);
	now_internal_draw:= false;
	//  
	reset_draw;
	//      
	SlideshotEMFCanvas.Destroy;
	CL_DrawerDelete (EmfDrawerID);
end;

//      
procedure TSliChart.CopyToClipboard ();
var
	SlideshotEMF: TMetaFile;
begin
	//  
	draw_metafile (SlideshotEMF);
	//     
	Clipboard.Clear;
	Clipboard.Assign (SlideshotEMF);
	//  
	SlideshotEMF.Destroy;
end;

//    
procedure TSliChart.Print ();
var
	SlideshotEMF: TMetaFile;
	k, kw, kh: double;
	Margin: integer;
begin
	//  
	draw_metafile (SlideshotEMF);
	//    
	Margin:= round (Printer.PageWidth*0.1);
	kw:= (Printer.PageWidth-2*Margin) / SlideshotEMF.Width;
	kh:= Printer.PageHeight / SlideshotEMF.Height;
	if kw<kh then k:=kw else k:=kh;
	SlideshotEMF.Width:= round (k * SlideshotEMF.Width);
	SlideshotEMF.Height:= round (k * SlideshotEMF.Height);
	//    
	Printer.Title:= 'ChartLib printing';
	Printer.BeginDoc;
	Printer.Canvas.Draw (Margin, Margin, SlideshotEMF);
	Printer.EndDoc;
	//  
	SlideshotEMF.Destroy;
end;

//    
procedure TSliChart.SaveAsEMF (const filename: string);
var
	SlideshotEMF: TMetaFile;
begin
	//  
	draw_metafile (SlideshotEMF);
	SlideshotEMF.SaveToFile (filename);
	//  
	SlideshotEMF.Destroy;
end;

//    
procedure TSliChart.SaveAsBMP (const filename: string);
var
	BMP : TBitMap;
	BmpDrawerID: integer;
begin
	//  
	BMP:= TBitMap.Create;
	BMP.Width:= ClientWidth;
	BMP.Height:= ClientHeight;
	//   ChartLib  
	ddef.drtype:= CL_DRAWER_WIN;
	ddef.hdc:= BMP.Canvas.Handle;
	BmpDrawerID:= CL_DrawerCreate (ddef);
	if BmpDrawerID<0 then exit;
	//CL_DrawerUse (BmpDrawerID);
	//  
	now_internal_draw:= true;
	CL_SlideRefresh (slide);
	last_error_code:= CL_SlideDraw (slide);
	now_internal_draw:= false;
	//  
	reset_draw;
	CL_DrawerDelete (BmpDrawerID);
	//    
	try
		BMP.SaveToFile (filename);
	finally
		BMP.Free;
	end;
end;

//    GIF
procedure TSliChart.SaveAsGIF (const filename: string);
var
	GifDrawerID: integer;
begin
	//   ChartLib  GIF
	ddef.drtype:= CL_DRAWER_GIF;
	ddef.filename:= PChar (filename);
	ddef.x1:= 0;
	ddef.y1:= 0;
	ddef.x2:= ClientWidth;
	ddef.y2:= ClientHeight;
	GifDrawerID:= CL_DrawerCreate (ddef);
	if GifDrawerID<0 then exit;
	//  
	now_internal_draw:= true;
	CL_SlideRefresh (slide);
	last_error_code:= CL_SlideDraw (slide);
	now_internal_draw:= false;
	//  
	CL_DrawerSave ();
	//  
	reset_draw;
	CL_DrawerDelete (GifDrawerID);
end;

// ---------------------------------------------------------------------
//     

//       
procedure TSliChart.ShowTable ();
var
	poinfo: TPointInfo;
begin
	if Assigned(grid) then begin
		poinfo.series:= 0;
		poinfo.point:= -2;
		grid_clear();
		while CL_ChartGetTableLine (slide,poinfo)=1 do begin
			if (poinfo.text<>nil) then
				grid_add_line (poinfo.text, poinfo.series, poinfo.point);
		end;
		grid.Refresh;
		if Assigned (FOnShowTable) then
    	FOnShowTable (Self);
	end;
end;

procedure TSliChart.grid_set_cell (r,c: integer; var S: string);
var
	w: integer;
begin
	if not Assigned(grid) then exit;
	if grid.ColCount<=c then grid.ColCount:= c+1;
	if grid.RowCount<=r then grid.RowCount:= r+1;
	grid.Cells[c,r]:= S;
	w:= grid.Canvas.TextWidth(S)+5;
	if grid.ColWidths[c]<w then grid.ColWidths[c]:= w;
end;

procedure TSliChart.grid_clear ();
var
	c, r: integer;
begin
	if not Assigned(grid) then exit;
	for r:= 0 to grid.RowCount-1 do
		for c:= 0 to grid.ColCount-1 do
			grid.Cells[c,r]:= '';
	grid.FixedCols:= 0;
	grid.ColCount:= 2;
	grid.RowCount:= 2;
	grid.FixedRows:= 1;
	grid_lines:= 0;
	grid_has_numbers:= false;
end;

procedure TSliChart.grid_add_line (line: PChar; nseries: integer; npoint: integer);
var
	c, r, tokLen: integer;
	token, s, buf: PChar;
	strbuf: string;
const
	delim = ';';
begin
	if not Assigned(grid) then exit;
	if (line=nil) or (line^=#0) then Exit;
	r:= grid_lines;
	c:= 0;
	if grid_lines=0 then
		grid_has_numbers:= (line^=delim);
	if grid_has_numbers and (grid_lines>0) then begin
		str (r, strbuf);
		grid_set_cell (r, c, strbuf);
		Inc(c);
	end;
	token:= line;
	repeat
		s:= StrScan (token,delim);
		if s=nil then
			tokLen:= StrLen (token)
		else
			tokLen:= s-token;
		GetMem (buf, tokLen+2);
		StrLCopy (buf, token, tokLen);
		strbuf:= StrPas(buf);
		grid_set_cell (r, c, strbuf);
		Inc (c);
		FreeMem (buf);
		token:= s+1;
	until s=nil;
	if grid_lines=0 then begin
		if grid_has_numbers then grid.FixedCols:= 1;
		grid.FixedRows:= 1;
	end;
	Inc (grid_lines);
	if (grid_lines>Length(grid_series_no)) then
		SetLength (grid_series_no, grid_lines+20);
	if (grid_lines>Length(grid_point_no)) then
		SetLength (grid_point_no, grid_lines+20);
	grid_series_no[grid_lines-1]:= nseries;
	grid_point_no[grid_lines-1]:= npoint;
end;

procedure TSliChart.grid_find_line (nseries: integer; npoint: integer);
var
	i: integer;
begin
	if not Assigned(grid) then exit;
	for i:=grid.FixedRows to grid_lines-1 do
		if (i<grid.RowCount) and (grid_series_no[i]=nseries) and (grid_point_no[i]=npoint) then begin
			grid.Row:= i;
			grid.Col:= grid.FixedCols;
			break;
		end;
end;

procedure TSliChart.GridSelectCell
	(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
	if not Assigned(grid) then exit;
	reset_draw;
	if CL_ChartGetActivePoint (slide, poinfo)=0 then begin
		poinfo.series:= grid_series_no[ARow];
		poinfo.point:= grid_point_no[ARow];
		CL_ChartSetActivePoint (slide, poinfo, 0);
	end;
end;

// ---------------------------------------------------------------------
//  

//        
procedure TSliChart.SetGrid (set_grid: TStringGrid);
begin
	grid:= set_grid;
	grid.OnSelectCell:= GridSelectCell;
end;

//    
procedure TSliChart.SetHasBuffer (set_buffered: boolean);
begin
	//  
	if buffered and not set_buffered then begin
  	buffer_free;
    buffered:= false;
	  if now_continue_draw then SlideRefresh;
	  end
	//  
  else if not buffered and set_buffered then begin
    buffered:= true;
		if now_continue_draw then SlideRefresh;
  	end;
	//   -    
	if Assigned(FOnPropertyChange)
  	then	FOnPropertyChange (self);
end;

//      
procedure TSliChart.SetHasStatus (set_hastatus: boolean);
begin
	hastatus:= set_hastatus;
	if hastatus then
		DataLine ('#StatusBar.Visible=on')
	else
		DataLine ('#StatusBar.Visible=off');
end;

//    
procedure TSliChart.SetSelectOutside (set_select_outside: boolean);
begin
	select_outside:= set_select_outside;
	if select_outside then
		DataLine ('#SelectOutside=on')
	else
		DataLine ('#SelectOutside=off');
	//   -    
	if Assigned(FOnPropertyChange)
  	then	FOnPropertyChange (self);
end;

//   
procedure TSliChart.SetSelectMode (set_select_mode: TSelectMode);
begin
	select_mode:= set_select_mode;
	StrFmt (cmd, '#SelectMode=%d', [Ord(select_mode)]);
	DataLine (cmd);
	//   -    
	if Assigned(FOnPropertyChange)
  	then	FOnPropertyChange (self);
end;

//   
procedure TSliChart.SetSelectShape (set_select_shape: TSelectShape);
begin
	select_shape:= set_select_shape;
	StrFmt (cmd, '#SelectShape=%d', [Ord(select_shape)]);
	DataLine (cmd);
	//   -    
	if Assigned(FOnPropertyChange)
  	then	FOnPropertyChange (self);
end;

//   
procedure SetLogName (filename: PChar);
begin
	CL_WorkDebug (filename);
end;

// ---------------------------------------------------------------------
// Public-

//  ()   
procedure TSliChart.DataFile (const FileName: PChar);
var
	info: TChartBackInfo;
begin
	//   -    
	info.text:= FileName;
	if Assigned(FOnDataFile) then
		FOnDataFile (self, info);
	//   CHD
	last_error_code:= CL_DataFile (slide, FileName);
end;

//       
procedure TSliChart.DataLine (const Line: PChar);
begin
	last_error_code:= CL_DataLine (slide, Line);
end;

//     
procedure TSliChart.DataVector (var Data: array of TInputData);
begin
	last_error_code:= CL_DataVector (slide, Data[0]);
end;

//    
procedure TSliChart.DataMatrix (rows: integer; fields: PChar; func: FMatrixInterface; obj: TObject);
begin
	last_error_code:= CL_DataMatrix (slide, rows, fields, func, obj);
end;

//        point     colname
procedure TSliChart.DataGet (Row: integer; ColName: PChar; var Data: TInputData);
begin
	last_error_code:= CL_DataGetValue (slide, Row, ColName, Data);
end;

//        point     colname
procedure TSliChart.DataPut (Row: integer; ColName: PChar; var Data: TInputData);
begin
	last_error_code:= CL_DataSetValue (slide, Row, ColName, Data);
end;

//      
procedure TSliChart.SlideSetRect ();
begin
	last_error_code:= CL_SlideSetRect (slide, 0, 0, ClientWidth-1, ClientHeight-1);
end;

//         
procedure TSliChart.SlideMaxZoom ();
begin
	last_error_code:= CL_SlideMaxZoom (slide);
end;

//      
procedure TSliChart.SlideDraw ();
begin
	if not Visible
		or (csDesigning in ComponentState)
		or (ClientWidth<2) or (ClientHeight<2) then Exit;
	//   -    
	if Assigned(FOnPaintStart) then
		FOnPaintStart (self);
  //  
	if buffered then begin
		buffer_create;
		reset_draw_buffer;
		end
  else
  	reset_draw_window;
	now_continue_draw:= true;
	CL_SlideRefresh (slide);
	CL_SlideBeginDraw (slide);
	//  
	while CL_SlideContinueDraw (slide, CL_STAGE_POINTS)=1 do;
  //      
  reset_draw_window;
	while CL_SlideContinueDraw (slide)=1 do;
	//  
	end_draw;
end;

//   
procedure TSliChart.SlideRefresh ();
begin
	if not Visible
		or (csDesigning in ComponentState)
		or (ClientWidth<2) or (ClientHeight<2) then Exit;
	//   -    
	if Assigned(FOnPaintStart) then
		FOnPaintStart (self);
	now_total_redraw:= true;
	//     
	if buffered then begin
		buffer_create;
		reset_draw_buffer;
		end
	else
		reset_draw_window;
	last_error_code:= CL_SlideRefresh (slide);
	CL_SlideBeginDraw (slide);
	//     
	continue_draw;
end;

//     
procedure TSliChart.ChartScreenToReal (x, y: integer; var PX, PY, PZ: double);
begin
	last_error_code:= CL_ChartScreenToReal (slide, x, y, PX, PY, PZ);
end;

//     
procedure TSliChart.ChartRealToScreen (xreal, yreal: double; var x, y: integer);
begin
	last_error_code:= CL_ChartRealToScreen (slide, xreal, yreal, x, y);
end;

//        
procedure TSliChart.ChartGetZoom (var x1: double; var y1: double; var x2: double; var y2: double);
begin
	last_error_code:= CL_ChartGetZoom (slide, x1, y1, x2, y2);
end;

//        
procedure TSliChart.ChartGetMaxZoom (var x1: double; var y1: double; var x2: double; var y2: double);
begin
	last_error_code:= CL_ChartGetMaxZoom (slide, x1, y1, x2, y2);
end;

//       
procedure TSliChart.ChartSetZoom (x1: double; y1: double; x2: double; y2: double);
begin
	last_error_code:= CL_ChartSetZoom (slide, x1, y1, x2, y2);
end;

//   stage   
procedure TSliChart.ChartDrawPoint (var info: TPointInfo; stage: integer);
begin
	last_error_code:= CL_ChartDrawPoint (slide, info, stage);
end;

//     
procedure TSliChart.ChartSelectPoint (var info: TPointInfo);
begin
	last_error_code:= CL_ChartSelectPoint (slide, info, Ord(select_mode));
end;

//     
procedure TSliChart.ChartSelectAll ();
begin
	last_error_code:= CL_ChartSelectAll (slide, Ord(select_mode));
end;

//     
procedure TSliChart.ChartSetActivePoint (var info: TPointInfo; dir: integer);
begin
	last_error_code:= CL_ChartSetActivePoint (slide, info, dir);
end;

//      
procedure TSliChart.ChartGetActivePoint (var info: TPointInfo);
begin
	last_error_code:= CL_ChartGetActivePoint (slide, info);
end;

//      
procedure TSliChart.ChartGetPointInfo (var info: TPointInfo);
begin
	last_error_code:= CL_ChartGetPointInfo (slide, info);
end;

//        
procedure TSliChart.ChartGetTableLine (var info: TPointInfo);
begin
	last_error_code:= CL_ChartGetTableLine (slide, info);
end;

//        series_name    (x,y)
procedure TSliChart.ChartGetXYInfo (x, y: integer; series, expr: PChar; var buffer: PChar);
begin
	last_error_code:= CL_ChartGetXYInfo (slide, x, y, series, expr, buffer);
end;

end.
