unit Main;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Printers, Clipbrd,
	Forms, Dialogs,	ChartLib, ExtCtrls;

type
	TfrmDrawer = class(TForm)
		procedure FormCreate(Sender: TObject);
		procedure FormPaint(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word;	Shift: TShiftState);
		procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure FormMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure FormDestroy(Sender: TObject);
	public
		WinDrawerID: integer;
		procedure DataFile (const FileName: string);
		procedure DataVector (var data: TInputData);
		procedure DataLine (data: PChar);
		procedure Test ();
	private
		slide: integer;
		curr_select_mode: integer;
		curr_select_shape: integer;
		now_continue_draw: boolean;
		now_resize: boolean;
		procedure reset_draw ();
		procedure begin_draw ();
		procedure continue_draw ();
		procedure end_draw ();
		procedure arrow_key (Key: Word);
		procedure draw_metafile (var SlideshotEMF: TMetaFile);
		procedure draw_clipboard ();
		procedure draw_printer ();
		procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
	end;

var
	frmDrawer: TfrmDrawer;

implementation

uses Table;

var
  cmd: array [0..300] of char;
	poinfo: TPointInfo;
	ddef: TDrawerDef;

{$R *.DFM}

procedure TfrmDrawer.FormCreate(Sender: TObject);
var
	p: integer;
begin
	//   ChartLib
	CL_WorkStart;
	CL_WorkDebug ('ChartLib.log');
	//   floating-point 
	Set8087CW($133F);
	//   
	now_continue_draw:= false;
	now_resize:= false;
	curr_select_mode:= 3;
	curr_select_shape:= 1;
	slide:= 0;
	WinDrawerID:= -1;
	//CL_DataLine (slide, '#NewSlide');
	//  ,    
	if ParamCount>0 then begin
		for p:=1 to ParamCount do
			DataFile (ParamStr(p));
		end
	else
		Test;
end;

procedure TfrmDrawer.FormDestroy(Sender: TObject);
begin
	//     ChartLib
	CL_WorkFinish;
end;

procedure TfrmDrawer.DataFile (const FileName: string);
begin
	reset_draw;
	if CL_DataFile (slide, PChar(FileName))<0 then Exit;
	//CL_ChartSetMaxZoom (slide);
	Caption:= FileName;
	begin_draw();
end;

procedure TfrmDrawer.DataLine (data: PChar);
begin
	CL_DataLine (slide, data);
end;

procedure TfrmDrawer.DataVector (var data: TInputData);
begin
	CL_DataVector (slide, data);
end;

//    
procedure TfrmDrawer.reset_draw ();
begin
	ddef.drtype:= CL_DRAWER_WIN;
	ddef.hdc:= Canvas.Handle;
	if WinDrawerID<0 then
		WinDrawerID:= CL_DrawerCreate (ddef);
	CL_DrawerUse (WinDrawerID);
	CL_DrawerUpdate (ddef);
end;

//  :    
procedure TfrmDrawer.begin_draw ();
begin
	reset_draw;
	CL_SlideSetRect (slide, 0, 0, ClientWidth-1, ClientHeight-1);
	CL_SlideRefresh (slide);
	CL_SlideBeginDraw (slide);
	continue_draw;
end;

//  :    WM_PAINT
procedure TfrmDrawer.continue_draw ();
begin
	now_continue_draw:= true;
	InvalidateRect (Handle, nil, FALSE);
end;

//  
procedure TfrmDrawer.end_draw();
begin
end;

//  Windows- "  "
procedure TfrmDrawer.WMEraseBkgnd (var Message: TWMEraseBkgnd);
begin
	begin_draw;
	Message.Result:= 1;
end;

//  Windows- " "
procedure TfrmDrawer.FormPaint (Sender: TObject);
begin
	if now_continue_draw then begin
		reset_draw;
		now_continue_draw:= (CL_SlideContinueDraw(slide)=1);
		if now_continue_draw then
			continue_draw
		else
			end_draw;
	end;
end;

function get_buttons (Shift: TShiftState): integer;
	var s: integer;
begin
	s:= 0;
	if ssShift  in Shift then s:= s+1;
	if ssAlt    in Shift then s:= s+2;
	if ssCtrl   in Shift then s:= s+4;
	if ssLeft   in Shift then s:= s+8;
	if ssRight  in Shift then s:= s+16;
	if ssMiddle in Shift then s:= s+32;
	if ssDouble in Shift then s:= s+64;
	get_buttons:= s;
end;

function get_key (key: Word): integer;
begin
	get_key:= 0;
	case Key of
		VK_LEFT  : get_key:= 1;
		VK_RIGHT : get_key:= 2;
		VK_UP    : get_key:= 3;
		VK_DOWN  : get_key:= 4;
		VK_PRIOR : get_key:= 5;
		VK_NEXT  : get_key:= 6;
		VK_HOME	 : get_key:= 7;
		VK_END   : get_key:= 8;
		VK_TAB   : get_key:= 9;
		VK_RETURN: get_key:= 10;
 		VK_INSERT: get_key:= 11;
 		VK_DELETE: get_key:= 12;
 		VK_BACK  : get_key:= 13;
 		VK_SPACE : get_key:= 14;
 		VK_ESCAPE: get_key:= 15;
		end;
end;

procedure TfrmDrawer.FormMouseDown
	(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	reset_draw;
	CL_OnMouseDown (slide, X, Y, get_buttons(Shift));
end;

procedure TfrmDrawer.FormMouseMove
	(Sender: TObject; Shift: TShiftState; X,Y: Integer);
begin
	reset_draw;
	CL_OnMouseMove (slide, X, Y, get_buttons(Shift));
	continue_draw;
end;

procedure TfrmDrawer.FormMouseUp
	(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var info: TPointInfo;
begin
	reset_draw;
	CL_OnMouseUp (slide, X, Y, get_buttons(Shift));
	if (CL_ChartGetActivePoint(slide,info)=0) then
		frmTable.FindLine (info.series, info.point);
	continue_draw;
end;

procedure TfrmDrawer.FormKeyDown
	(Sender: TObject; var Key: Word; Shift: TShiftState);
var
	slide_old: integer;
begin
	slide_old:= slide;
	reset_draw;
	if CL_OnKeyPress (slide, get_key(Key), get_buttons(Shift))=1 then begin
		continue_draw;
		Exit;
	end;
	if (Shift=[ssCtrl]) then case Key of
	ord('R') : begin
		begin_draw;
		end;
	ord('L') : begin   //  
		if (ParamCount>0) then
			DataFile (ParamStr(1))
		end;
	ord('A') : begin
		CL_ChartSelectAll (slide, curr_select_mode);
		end;
	ord('M') : begin
		curr_select_mode:= (curr_select_mode mod 3) + 1;
		StrFmt (cmd, '#SelectMode=%d', [curr_select_mode]);
		CL_DataLine (slide, cmd);
		end;
	ord('S') : begin
		curr_select_shape:= (curr_select_shape mod 4) + 1;
		StrFmt (cmd, '#SelectShape=%d', [curr_select_shape]);
		CL_DataLine (slide, cmd);
		end;
	ord('T') : begin
		poinfo.series:= 0;
		poinfo.point:= -2;
		frmTable.Clear();
		frmTable.slide:= slide;
		while CL_ChartGetTableLine (slide,poinfo)=1 do begin
			if (poinfo.text<>nil) then
				frmTable.AddLine (poinfo.text, poinfo.series, poinfo.point);
			end;
		frmTable.OptimizeWindowSize;
		frmTable.Show;
		end;
	ord('C'), VK_INSERT : begin
		draw_clipboard ();
		end;
	ord('P') : begin
		draw_printer ();
		end;
	VK_NEXT : begin  // PageDown
		CL_DataLine (slide, '#Slide <');
		if slide<>slide_old then
			begin_draw;
		end;
	VK_PRIOR : begin // PageUp
		CL_DataLine (slide, '#Slide >');
		if slide<>slide_old then
			begin_draw;
		end;
	end;
	if (Shift=[]) then case Key of
		VK_RIGHT, VK_LEFT, VK_UP, VK_DOWN:
			arrow_key (Key);
	end;
end;

procedure TfrmDrawer.arrow_key (Key: Word);
var dir, res: integer;
begin
	if CL_ChartGetActivePoint (slide,poinfo)=0 then begin
		dir:= 0;
		case Key of
			VK_RIGHT : dir:= +1;
			VK_LEFT  : dir:= -1;
			VK_UP    : dir:= +2;
			VK_DOWN  : dir:= -2;
		end;
		res:= CL_ChartSetActivePoint (slide, poinfo, dir);
		if res=0 then
			frmTable.FindLine (poinfo.series, poinfo.point);
	end;
end;

procedure TfrmDrawer.draw_metafile (var SlideshotEMF: TMetaFile);
var
	SlideshotEMFCanvas: TMetafileCanvas;
	EmfDrawerID: integer;
begin
	//  
	SlideshotEMF:= TMetafile.Create;
	SlideshotEMF.Width:= ClientWidth;
	SlideshotEMF.Height:= ClientHeight;
	//   
	SlideshotEMFCanvas:= TMetafileCanvas.Create (SlideshotEMF,0);
	//   ChartLib   
	ddef.drtype:= CL_DRAWER_EMF;
	ddef.hdc:= SlideshotEMFCanvas.Handle;
	EmfDrawerID:= CL_DrawerCreate (ddef);
	CL_DrawerUse (EmfDrawerID);
	//  
	CL_DataLine (slide, '#StatusBar.Visible= off');
	CL_SlideRefresh (slide);
	CL_SlideDraw (slide);
	CL_DataLine (slide, '#StatusBar.Visible= on');
	//  
	reset_draw;
	//      
	//SlideshotEMF.SaveToFile ('test.emf');
	SlideshotEMFCanvas.Destroy;
	CL_DrawerDelete (EmfDrawerID);
end;

procedure TfrmDrawer.draw_clipboard ();
var
	SlideshotEMF: TMetaFile;
begin
	//  
	draw_metafile (SlideshotEMF);
	//     
	Clipboard.Clear;
	Clipboard.Assign (SlideshotEMF);
	//  
	SlideshotEMF.Destroy;
end;

procedure TfrmDrawer.draw_printer ();
var
	SlideshotEMF: TMetaFile;
	k, kw, kh: double;
	Margin: integer;
begin
	//  
	draw_metafile (SlideshotEMF);
	//    
	Margin:= round (Printer.PageWidth*0.1);
	kw:= (Printer.PageWidth-2*Margin) / SlideshotEMF.Width;
	kh:= Printer.PageHeight / SlideshotEMF.Height;
	if kw<kh then k:=kw else k:=kh;
	SlideshotEMF.Width:= round (k * SlideshotEMF.Width);
	SlideshotEMF.Height:= round (k * SlideshotEMF.Height);
	//    
	Printer.Title:= 'ChartLib printing';
	Printer.BeginDoc;
	Printer.Canvas.Draw (Margin, Margin, SlideshotEMF);
	Printer.EndDoc;
	//  
	SlideshotEMF.Destroy;
end;

procedure TfrmDrawer.Test ();
	var i, N: integer;
     		data: array [0..10] of TInputData;
     		d: TInputData;
     		x, y: double;
begin
	DataLine ('#NewPanel viaDataLine');
	DataLine ('#Title CHD viewer test: 10000 points');
	DataLine ('#XName x');
	DataLine ('#YName y');
	DataLine ('#Format X Y W=0 LW=2 LC=2 LG=8 LI=(sin(X*6))');
	N:= 5000;
	//     -   
	for i:= 1 to N do begin
		x:= sin (i/45.8);
		y:= cos (i/27.2);
		DataLine (PChar(Format('%0.5f,%0.5f',[x,y])));
		end;
	//     -   
	DataLine ('#NewPanel viaDataVector');
	DataLine ('#XName x');
	DataLine ('#YName y');
	DataLine ('#Format X Y W{PointSize} S=19 LW=2 LC{LineColor}=22 LG=4 LI=(sin(X*6))');
	data[0].dType:= CL_DATA_REAL;
	data[1].dType:= CL_DATA_REAL;
	data[2].dType:= CL_DATA_BYTE;
	data[3].dType:= 0;  // =    
	for i:= 1 to N do begin
		data[0].dReal:= sin (i/41.8);
		data[1].dReal:= cos (i/27.2);
		data[2].dByte:= 0;
		DataVector (data[0]);
		end;
	//     
	d.dType:= CL_DATA_REAL;
	d.dReal:= 30;
	CL_DataSetValue (slide, 234, 'PointSize', d);
	//      
	d.dType:= CL_DATA_BYTE;
	d.dByte:= 1;  //  -  
	CL_DataSetValue (slide, 0, 'LineColor', d);
	//    
	CL_SlideMaxZoom (slide);
	begin_draw;
end;

end.
