unit Table;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	Consts, Grids, ChartLib{, DataFile};

type
	TfrmTable = class(TForm)
		grdMain: TStringGrid;
		procedure grdMainSelectCell
			(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
	private
		{
		fc: integer;
		fr: integer;
		}
		SeriesNo: array of integer;
		PointNo: array of integer;
		nLines: integer;
		has_numbers: boolean;
		procedure set_grid_cell (r,c: integer; var S: string);
	public
		slide: integer;
		{
		procedure ReadCmdline;
		procedure LoadFile (const FileName: string);
		}
		procedure Clear ();
		procedure AddLine (line: PChar; nseries: integer; npoint: integer);
		procedure FindLine (nseries: integer; npoint: integer);
		procedure OptimizeWindowSize ();
	end;

var
	frmTable: TfrmTable;

implementation

uses Main;
{$R *.DFM}

type TScrollStyle = (ssNone, ssHorizontal, ssVertical, ssBoth);

procedure TfrmTable.set_grid_cell (r,c: integer; var S: string);
	var w: integer;
begin
	if grdMain.ColCount<=c then grdMain.ColCount:= c+1;
	if grdMain.RowCount<=r then grdMain.RowCount:= r+1;
	grdMain.Cells[c,r]:= S;
	w:= grdMain.Canvas.TextWidth(S)+5;
	if grdMain.ColWidths[c]<w then grdMain.ColWidths[c]:= w;
end;

procedure TfrmTable.Clear ();
var c, r: integer;
begin
	for r:= 0 to grdMain.RowCount-1 do
		for c:= 0 to grdMain.ColCount-1 do
			grdMain.Cells[c,r]:= '';
	grdMain.FixedCols:= 0;
	grdMain.ColCount:= 2;
	grdMain.RowCount:= 2;
	grdMain.FixedRows:= 1;
	nLines:= 0;
	has_numbers:= false;
	slide:= 0;
end;

procedure TfrmTable.AddLine (line: PChar; nseries: integer; npoint: integer);
var
	c, r, tokLen: integer;
	token, s, buf: PChar;
	strbuf: string;
const
	delim = ';';
begin
	if (line=nil) or (line^=#0) then Exit;
	r:= nLines;
	c:= 0;
	if nLines=0 then
		has_numbers:= (line^=delim);
	if has_numbers and (nLines>0) then begin
		str (r, strbuf);
		set_grid_cell (r, c, strbuf);
		Inc(c);
	end;
	token:= line;
	repeat
		s:= StrScan (token,delim);
		if s=nil then
			tokLen:= StrLen (token)
		else
			tokLen:= s-token;
		GetMem (buf, tokLen+2);
		StrLCopy (buf, token, tokLen);
		strbuf:= StrPas(buf);
		set_grid_cell (r, c, strbuf);
		Inc (c);
		FreeMem (buf);
		token:= s+1;
	until s=nil;
	if nLines=0 then begin
		if has_numbers then grdMain.FixedCols:= 1;
		grdMain.FixedRows:= 1;
	end;
	Inc (nLines);
	if (nLines>Length(SeriesNo)) then
		SetLength (SeriesNo, nLines+20);
	if (nLines>Length(PointNo)) then
		SetLength (PointNo, nLines+20);
	SeriesNo[nLines-1]:= nseries;
	PointNo[nLines-1]:= npoint;
end;

procedure TfrmTable.FindLine (nseries: integer; npoint: integer);
var
	i: integer;
begin
	for i:=grdMain.FixedRows to nLines-1 do
		if (i<grdMain.RowCount) and (SeriesNo[i]=nseries) and (PointNo[i]=npoint) then begin
			grdMain.Row:= i;
			grdMain.Col:= grdMain.FixedCols;
			break;
		end;
end;

procedure TfrmTable.OptimizeWindowSize ();
	var c, w, wmax: integer;
begin
	w:= 0;
	for c:= 0 to grdMain.ColCount-1 do
		w:= w + grdMain.ColWidths[c] + grdMain.GridLineWidth;
	wmax:= Round(Screen.Width*0.8);
	if w>wmax then w:= wmax;
	ClientWidth:= w + 17;
end;

procedure TfrmTable.grdMainSelectCell
	(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
var
	poinfo: TPointInfo;
  ddef: TDrawerDef;
begin
	ddef.drtype:= CL_DRAWER_WIN;
	ddef.hdc:= frmDrawer.Canvas.Handle;
	CL_DrawerUpdate (ddef);
	if CL_ChartGetActivePoint (slide, poinfo)=0 then begin
  	poinfo.series:= SeriesNo[ARow];
    poinfo.point:= PointNo[ARow];
		CL_ChartSetActivePoint (slide, poinfo, 0);
	end;
end;

(*
procedure TfrmTable.ReadCmdline;
	var P: integer;
		opt: string;
		foo: integer;
begin
	if ParamCount>0 then
		begin
		fr:= 1;
		fc:= 1;
		for P:= 2 to ParamCount do
			begin
			opt:= copy(ParamStr(P),1,3);
			if opt='-fc' then val(copy(ParamStr(P),4,255), fc, foo);
			if opt='-fr' then val(copy(ParamStr(P),4,255), fr, foo);
			end;
		grdMain.ColCount:= fc+1;
		grdMain.RowCount:= fr+1;
		LoadFile (ParamStr(1));
		grdMain.FixedCols:= fc;
		grdMain.FixedRows:= fr;
		OptimizeWindowSize;
		end
	else
		ErrorMsg ('USAGE: TableDIT <FileName> [-fc<FixedCols>] [-fr<FixedRows>]');
end;

procedure TfrmTable.LoadFile (const FileName: string);
	var txt: TextFile;
		c, r: integer;
		S, Sr: string;
begin
	{$I-}
	AssignFile(txt, FileName);
	Reset(txt);
	{$I+}
	if (IOResult = 0) and (FileName <> '') then
		begin
		if grdMain.ColCount<2 then grdMain.ColCount:= 2;
		if grdMain.RowCount<2 then grdMain.RowCount:= 2;
		grdMain.FixedCols:= 1;
		grdMain.FixedRows:= 1;
		c:= 0; r:= 0;
		repeat
			ReadCell (txt, S, ';');
			if (LastReadCellState=gsEof) and (Length(S)=0) then Break;
			if (c=0) and (r>0) then begin
				str(r,Sr);
				set_grid_cell (r,c,Sr);
				c:= 1;
				end;
			set_grid_cell (r,c,S);
			Inc(c);
			if LastReadCellState=gsEoln then begin
				c:= 0; Inc(r);
				end;
		until LastReadCellState = gsEof;
		CloseFile(txt);
		grdMain.RowCount:= r;
		end
	else
		ErrorMsg (PChar('Can not open file "'+FileName+'"'));
end;
*)

end.



